﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace TGC.RadioButtonUtil
{
    /// <summary>
    /// RadioButtonContainer class provides the usual radio button group functionality.
    /// </summary>
    /// <remarks>
    /// Radio buttons are added as any other control; however, this class provides specialized
    /// methods that deal with radio buttons. The AutoCheck property of any child radio button 
    /// control is ignored, and it is not altered in any way. The radio buttons are accessed via 
    /// their indices using the <code>GetRadioButtonAt()</code> method, or the array returned by 
    /// the <code>RadioButtons</code> property. The buttons are sorted by tab index. 
    /// NOTE that addition/removal of radio buttons at runtime might alter the indices of other 
    /// contained radio buttons.
    /// </remarks>
    [Designer(typeof(Design.RadioButtonContainerDesigner))]
    public partial class RadioButtonContainer : UserControl
    {
        private RadioButtonGroup internalGroup = new RadioButtonGroup();
        private int defaultOptionIndex = 0;

        public RadioButtonContainer()
        {
            InitializeComponent();
            internalGroup.TabOrderChanged += new EventHandler(internalGroup_TabOrderChanged);
            internalGroup.StateChanged += new EventHandler<RadioBtnGroupEventArgs>(internalGroup_StateChanged);
        }

        private void internalGroup_StateChanged(object sender, RadioBtnGroupEventArgs e)
        {
            OnStateChanged();
        }

        private void internalGroup_TabOrderChanged(object sender, EventArgs e)
        {
            OnTabOrderChanged();
        }

        protected override void OnLoad(EventArgs e)
        {
            // This insures that the control behaves right if the tab order
            // of existing radioButton-s is altered in the designer.

            if (internalGroup.Count > 1)
                internalGroup.Sort();

            internalGroup[defaultOptionIndex].Checked = true;

            base.OnLoad(e);
        }

        protected override void OnControlAdded(ControlEventArgs e)
        {
            base.OnControlAdded(e);

            RadioButton rBtn = e.Control as RadioButton;
            if (rBtn != null)
                internalGroup.Add(rBtn);
        }

        protected override void OnControlRemoved(ControlEventArgs e)
        {
            base.OnControlRemoved(e);

            RadioButton rBtn = e.Control as RadioButton;
            if (rBtn != null)
                internalGroup.Remove(rBtn);
        }

        /// <summary>
        /// Gets an array containing all the radio buttons contained in this control.
        /// </summary>
        public RadioButton[] RadioButtons
        {
            get { return internalGroup.Buttons; }
        }

        /// <summary>
        /// Returns currently selected radio button, or null if no option is checked.
        /// </summary>
        public RadioButton CurrentOption
        {
            get { return internalGroup.Current; }
        }

        public int CurrentOptionIndex
        {
            get { return internalGroup.CurrentIndex; }
        }

        /// <summary>
        /// Gets a value indicating if any of the options is selected.
        /// </summary>
        public bool IsOptionSelected
        {
            get { return internalGroup.Current != null; }
        }

        /// <summary>
        /// Gets the number of radio buttons that are parented by this control.
        /// </summary>
        public int RadioButtonCount
        {
            get { return internalGroup.Count; }
        }

        /// <summary>
        /// Returns the radio button at the given index.
        /// </summary>
        /// <param name="index">The index of the radio button to return</param>
        /// <returns>A <code>RadioButton</code> instance.</returns>
        /// <exception cref="System.IndexOutOfRangeException"></exception>
        public RadioButton GetRadioButtonAt(int index)
        {
            if (index < 0 || index >= internalGroup.Count)
                throw new IndexOutOfRangeException(
                    "Current: " + index.ToString() + ", min: 0, max: " + (internalGroup.Count - 1).ToString()
                    );

            return internalGroup[index];
        }

        public int DefaultOptionIndex
        {
            get { return defaultOptionIndex; }

            set
            {
                if (value < 0 || (internalGroup.Count > 0 && value >= internalGroup.Count))
                    throw new IndexOutOfRangeException();

                defaultOptionIndex = value;
            }
        }

        public void DeselectAllOptions()
        {
            internalGroup.DeselectAll();
        }

        public void Sort()
        {
            internalGroup.Sort();
        }

        /// <summary>
        /// Raised when the tab order of child radio button controls changes. Call Sort() to reorder the radio buttons.
        /// </summary>
        public event EventHandler TabOrderChanged;

        private void OnTabOrderChanged()
        {
            if (TabOrderChanged != null)
                TabOrderChanged(this, new EventArgs());
        }

        /// <summary>
        /// State changed event is rased when the option selection state is changed, i.e. when client code
        /// selects an option or when all choices are cleared.
        /// </summary>
        public event EventHandler<RadioBtnContainerEventArgs> StateChanged;

        private void OnStateChanged()
        {
            if (StateChanged != null)
            {
                StateChanged(this, RadioBtnContainerEventArgs.CreateFromContainer(this));
            }
        }

        internal RadioButtonGroup InternalGroup
        {
            get { return internalGroup; }
        }
    }
}
